// ------------------------------------------------------------------------------------------------------
// LightningChart® example code: Demo shows Signal Filters IIR/FIR in use.
//
// If you need any assistance, or notice error in this example code, please contact support@lightningchart.com. 
//
// Permission to use this code in your application comes with LightningChart® license. 
//
// https://lightningchart.com | support@lightningchart.com | sales@lightningchart.com
//
// © LightningChart Ltd 2009-2023. All rights reserved.  
// ------------------------------------------------------------------------------------------------------
using Arction.Wpf.Charting;
using Arction.Wpf.Charting.Axes;
using Arction.Wpf.Charting.SeriesXY;
using Arction.Wpf.Charting.Views.ViewXY;
using Arction.Wpf.SignalProcessing;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Globalization;
using System.Linq;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Media;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace InteractiveExamples
{
    public partial class ExampleSignalFilters : Window, IDisposable
    {
        public static readonly DependencyProperty GenVisibilityProperty =
      DependencyProperty.Register(
          "GenVisibility",
          typeof(Visibility),
          typeof(ExampleSignalFilters),
          new PropertyMetadata(Visibility.Visible)
      );

        public Visibility GenVisibility
        {
            get { return (Visibility)GetValue(GenVisibilityProperty); }
            set { SetValue(GenVisibilityProperty, value); }
        }

        private delegate void ChartUpdater(double[] newSamples, double[] filteredSamples, AreaSeriesPoint[][] spectrumLinePoints);
        //The chart
        private LightningChart _Chart = new LightningChart();
        //The chart
        private LightningChart _ChartArea = new LightningChart();
        const int _spectrumWindowLength = 4000;
        IIRFilter _IIRFilter;
        FIRFilter _FIRFilter;
        bool FirIsInUse = true;
        SignalGenerator _SignalGenerator;
        // Thread-fed samples must be fed to the chart through UI invoke. Samples for AudioOutput doesn't have to be invoked.
        private ChartUpdater _chartUpdater;
        /// <summary>
        /// FFT spectrum calculator. Part of Arction Signal Processing tool.
        /// </summary>
        private SpectrumCalculator _spectrumCalculator;
        /// <summary>
        /// "smooth" factor of spectrum (moving average window length)
        /// </summary>
        private int _spectrumSmoothLength = 10;
        /// <summary>
        /// Raw data sample count
        /// </summary>
        private int _rawDataSampleCount = 0;
        /// <summary>
        /// Buffer for latest data FFT
        /// </summary>
        private double[] _latestDataForFFT = new double[_spectrumWindowLength];
        /// <summary>
        /// Buffer for latest filtered data for FFT
        /// </summary>
        private double[] _latestFilteredDataForFFT = new double[_spectrumWindowLength];
        /// <summary>
        /// Thread for do zoom operation in spectrum
        /// </summary>
        Thread _ZoomSpectrumthread;
        /// <summary>
        /// Highest spectrum spike for zooming porpoise
        /// </summary>
        private double _highestSpectrumSpike = 0;
        /// <summary>
        /// Signal filters constructor
        /// </summary>
        public ExampleSignalFilters()
        {
            InitializeComponent();
            //Generate Signal generator, filters and spectrum
            _SignalGenerator = new SignalGenerator();
            _SignalGenerator.ThreadType = ThreadType.Thread;
            _SignalGenerator.OutputInterval = 0; 
            _IIRFilter = new IIRFilter();
            _FIRFilter = new FIRFilter();
            _spectrumCalculator = new SpectrumCalculator();
            _FIRFilter.SetFactors(_FirLowPass);
            //Create chart
            CreateChart();
            _chartUpdater = new ChartUpdater(FeedSamplesToChart);
            ChartGrid.Children.Add(_Chart);
            ChartAreaGrid.Children.Add(_ChartArea);
            _SignalGenerator.DataGenerated += _SignalGenerator_DataGenerated;
            _SignalGenerator.Start();
            ViewXY_Zoomed(_Chart.ViewXY,new ZoomedXYEventArgs() { 
                XRanges = new List<RangeInfo>{new RangeInfo() {
                    CurrentMax = _Chart.ViewXY.XAxes[0].Maximum,
                    CurrentMin = _Chart.ViewXY.XAxes[0].Minimum,
                    Axis = _Chart.ViewXY.XAxes[0] } } });
            //Fit signals and spectrum
            FitSignalsY();
            FitSpectrumY();
        }

        /// <summary>
        /// Signal data generated event
        /// </summary>
        /// <param name="args">generated data</param>
        private void _SignalGenerator_DataGenerated(DataGeneratedEventArgs args)
        {
            //Make filtering and FFT computation in this Signal Generator's background thread. 
            double[] newSamples = args.Samples[0];
            int countOfNewSamples = newSamples.Length;

            double[] filteredSamples;

            //Make the filtering with FIR or IIR filter
            if (FirIsInUse)
            {
                _FIRFilter.FilterData(newSamples, out filteredSamples);
            }
            else
            {
                if (!_IIRFilter.FilterData(newSamples, out filteredSamples))
                {
                    Debug.WriteLine("UnstableFilter");
                }
            }

            double[] filteredSamplesLengthMatched = filteredSamples; 

            if (filteredSamples.Length < newSamples.Length)
            {
                //Add padding bytes in the beginning, as the filter hasn't initialized yet. 

                int padding = newSamples.Length - filteredSamples.Length;
                filteredSamplesLengthMatched = new double[countOfNewSamples];
                Array.Copy(filteredSamples, 0, filteredSamplesLengthMatched, padding, filteredSamples.Length); 
            }

            int latestDataBuffLen = _latestDataForFFT.Length;
            if (countOfNewSamples >= latestDataBuffLen)
            {
                //Take the latest samples only in the buffer. 
                //Raw 
                Array.Copy(newSamples, countOfNewSamples - latestDataBuffLen, _latestDataForFFT, 0, latestDataBuffLen);

                //Filtered 
                Array.Copy(filteredSamplesLengthMatched, countOfNewSamples - latestDataBuffLen, _latestFilteredDataForFFT, 0, latestDataBuffLen);

            }
            else
            {
                //Append to the tail 
                double[] newArray = new double[latestDataBuffLen]; 
                Array.Copy(_latestDataForFFT, countOfNewSamples, newArray, 0, latestDataBuffLen - countOfNewSamples);
                Array.Copy(newSamples, 0, newArray, latestDataBuffLen - countOfNewSamples, countOfNewSamples);
                _latestDataForFFT = newArray;


                double[] newFiltered = new double[latestDataBuffLen];
                Array.Copy(_latestFilteredDataForFFT, countOfNewSamples, newFiltered, 0, latestDataBuffLen - countOfNewSamples);
                Array.Copy(filteredSamplesLengthMatched, 0, newFiltered, latestDataBuffLen - countOfNewSamples, countOfNewSamples);
                _latestFilteredDataForFFT = newFiltered;
            }
            
            AreaSeriesPoint[][] spectrumLinePoints = new AreaSeriesPoint[2][];

            Parallel.For(0, 2, (i) =>
            {
                // Calculate raw signal data FFT and fill FFT monitor. 
                double[] fft;
                int processedSampleCount = 0;
                double[] samplesForFFT;
                int fftComputationLength = _spectrumWindowLength; 

                if (i == 0)
                    samplesForFFT = _latestDataForFFT;
                else
                    samplesForFFT = _latestFilteredDataForFFT; 

                bool calculated = _spectrumCalculator.PowerSpectrumOverlapped(samplesForFFT, fftComputationLength, 50, out fft, out processedSampleCount);
                int fftDataLen = fft.Length;
                double xStep = 40000 / 2.0 / (fftDataLen - 1);

                double[] fftmoving = FilterRoutines.MovingAverage(fft, _spectrumSmoothLength);

                //Take frequency content of only up to 5000 Hz. 
                int areaPointsCount = fftDataLen / 4 + 1;

                // Convert given FFT data points to compatible for AreaSeries format.
                AreaSeriesPoint[] asp = new AreaSeriesPoint[areaPointsCount];

                for (int dataPointIndex = 0; dataPointIndex < areaPointsCount; dataPointIndex++)
                {
                    asp[dataPointIndex].X = dataPointIndex * xStep;
                    asp[dataPointIndex].Y = fftmoving[dataPointIndex];
                    if(i == 0 && fftmoving[dataPointIndex] > _highestSpectrumSpike)
                        _highestSpectrumSpike = fftmoving[dataPointIndex];
                }
                spectrumLinePoints[i] = asp;
            });


            //Update processed data to the charts
            if(_chartUpdater != null)
                Dispatcher.Invoke(_chartUpdater, newSamples, filteredSamplesLengthMatched, spectrumLinePoints);

        }

        /// <summary>
        /// Put samples to chart
        /// </summary>
        /// <param name="newSamples">Raw new samples</param>
        /// <param name="filteredSamples">Filtered new samples</param>
        /// <param name="spectrumLinePoints">Spectrum points to spectrum chart</param>
        private void FeedSamplesToChart(double[] newSamples, double[] filteredSamples, AreaSeriesPoint[][] spectrumLinePoints)
        {
            if (_Chart == null)
            {
                return;
            }

            int countOfNewSamples = newSamples.Length;
            int countOfFilteredSamples = filteredSamples.Length; 

            _Chart.BeginUpdate();
            _ChartArea.BeginUpdate();


            //Append new samples to tail of the series

            float[] samples = new float[countOfNewSamples];
            for (int i = 0; i < countOfNewSamples; i++)
            {
                samples[i] = (float)newSamples[i];
            }

            float[] filtered = new float[countOfFilteredSamples];
            for (int i = 0; i < countOfFilteredSamples; i++)
            {
                filtered[i] = (float)filteredSamples[i];
            }

            _Chart.ViewXY.SampleDataBlockSeries[0].AddSamples(samples, false);

            _rawDataSampleCount += countOfNewSamples;


            _Chart.ViewXY.SampleDataBlockSeries[1].AddSamples(filtered, false);
            
            //Update X axis scroll position 
            _Chart.ViewXY.XAxes[0].ScrollPosition = (double)(_rawDataSampleCount-1) / 40000.0;

            
            for (int i = 0; i < 2; i++)
            {
                // Set calculated FFT data to FFT monitor.
                _ChartArea.ViewXY.AreaSeries[i].Points = spectrumLinePoints[i];
            }

            _Chart.EndUpdate();
            _ChartArea.EndUpdate(); 
        }

        /// <summary>
        /// Create charts
        /// </summary>
        private void CreateChart()
        {
            //Signal chart
            _ChartArea.BeginUpdate();
            _Chart.BeginUpdate();

            _Chart.ViewXY.XAxes[0].ScrollMode = XAxisScrollMode.Scrolling;
            _Chart.ViewXY.XAxes[0].ZoomingEnabled = true;
            _Chart.ViewXY.XAxes[0].PanningEnabled = true;
            _Chart.ViewXY.XAxes[0].SetRange(-1.0, 0);
            _Chart.ViewXY.XAxes[0].Visible = false; 

            AxisX xAxis2 = new AxisX(_Chart.ViewXY);
            xAxis2.Title.Text = "Time (s)";
            xAxis2.PanningEnabled = false;
            xAxis2.MajorGrid.Visible = false;
            xAxis2.MinorGrid.Visible = false;
            _Chart.ViewXY.XAxes.Add(xAxis2);

            _Chart.ViewXY.YAxes.Add(new AxisY());
            _Chart.ViewXY.XAxes[0].Title.Text = "time (s)";
            _Chart.ViewXY.YAxes[0].SetRange(-1200, 1200);
            _Chart.ViewXY.YAxes[0].Title.Text = "Raw signal (mV)";
            _Chart.ViewXY.YAxes[1].Title.Text = "Filtered signal (mV)";
            _Chart.ViewXY.YAxes[1].SetRange(-1200, 1200);
            _Chart.ViewXY.AxisLayout.YAxesLayout = YAxesLayout.Stacked;
            _Chart.ViewXY.AxisLayout.XAxisAutoPlacement = XAxisAutoPlacement.Off;
            _Chart.ViewXY.LegendBoxes[0].Visible = false;
            _Chart.Title.Text = "Signal";
            _Chart.ChartName = "Signal Filter";
            _Chart.ViewXY.DropOldSeriesData = true;
            _Chart.ViewXY.Zoomed += ViewXY_Zoomed;

            //Raw SDBS
            SampleDataBlockSeries SDSRaw = new SampleDataBlockSeries(_Chart.ViewXY, _Chart.ViewXY.XAxes[0], _Chart.ViewXY.YAxes[0]);
            SDSRaw.SamplingFrequency = 40000;
            SDSRaw.FirstSampleTimeStamp = 0;
            SDSRaw.Color = Colors.Orange;
            SDSRaw.AssignYAxisIndex = 0;
            SDSRaw.AllowUserInteraction = false;
            SDSRaw.Highlight = Highlight.None;
            _Chart.ViewXY.SampleDataBlockSeries.Add(SDSRaw);

            //Filtered SDBS
            SampleDataBlockSeries SDSFiltered = new SampleDataBlockSeries(_Chart.ViewXY, _Chart.ViewXY.XAxes[0], _Chart.ViewXY.YAxes[1]);
            SDSFiltered.SamplingFrequency = 40000;
            SDSFiltered.FirstSampleTimeStamp = 0;
            SDSFiltered.Color = Colors.Yellow;
            SDSFiltered.AssignYAxisIndex=1;
            SDSFiltered.AllowUserInteraction =false;
            SDSFiltered.Highlight = Highlight.None;
            _Chart.ViewXY.SampleDataBlockSeries.Add(SDSFiltered);

            //Power spectrum chart
            _ChartArea.Title.Text = "Spectrum";
            _ChartArea.ViewXY.LegendBoxes[0].Visible = false;
            _ChartArea.ChartName = "Signal Filter";
            _ChartArea.ViewXY.XAxes[0].ScrollMode = XAxisScrollMode.None;
            _ChartArea.ViewXY.XAxes[0].ZoomingEnabled = true;
            _ChartArea.ViewXY.XAxes[0].PanningEnabled = true;
            _ChartArea.ViewXY.XAxes[0].SetRange(0, 5000);
            _ChartArea.ViewXY.AxisLayout.XAxisAutoPlacement = XAxisAutoPlacement.Off;
            _ChartArea.ViewXY.XAxes[0].Title.Text = "Frequency (Hz)";
            _ChartArea.ViewXY.YAxes[0].SetRange(0, 120000);
            _ChartArea.ViewXY.YAxes.Add(new AxisY());
            _ChartArea.ViewXY.YAxes[0].Title.Text = "P(f)";
            _ChartArea.ViewXY.YAxes[1].Title.Text = "P(f)";
            _ChartArea.ViewXY.AxisLayout.YAxesLayout = YAxesLayout.Stacked;
            _ChartArea.ViewXY.YAxes[1].SetRange(0, 120000);
            _ChartArea.ViewXY.YAxes[0].RangeChanged += SpectrumYAxisRangeChange; 
            _ChartArea.ViewXY.YAxes[1].RangeChanged += SpectrumYAxisRangeChange;

            // Create Area series to represent calculated FFT for a given signal.
            AreaSeries UnfilteredAreaSeries = new AreaSeries(_ChartArea.ViewXY, _ChartArea.ViewXY.XAxes[0], _ChartArea.ViewXY.YAxes[0])
            {
                AllowUserInteraction = false
            };
            UnfilteredAreaSeries.LineStyle.Color = SDSRaw.Color;
            UnfilteredAreaSeries.LineStyle.Width = 1;
            UnfilteredAreaSeries.Fill.Color = ChartTools.CalcGradient(UnfilteredAreaSeries.LineStyle.Color, Colors.Black, 50);
            UnfilteredAreaSeries.Fill.GradientFill = GradientFill.Solid;
            UnfilteredAreaSeries.Title.Text = "P(f)";
            UnfilteredAreaSeries.LimitYToStackSegment = true;
            _ChartArea.ViewXY.AreaSeries.Add(UnfilteredAreaSeries);

            // Create Area series to represent calculated FFT for a given signal.
            AreaSeries FilteredAreaSeries = new AreaSeries(_ChartArea.ViewXY, _ChartArea.ViewXY.XAxes[0], _ChartArea.ViewXY.YAxes[1])
            {
                AllowUserInteraction = false
            };
            FilteredAreaSeries.LineStyle.Color = SDSFiltered.Color;
            FilteredAreaSeries.LineStyle.Width = 1;
            FilteredAreaSeries.Fill.Color = ChartTools.CalcGradient(UnfilteredAreaSeries.LineStyle.Color, Colors.Black, 50);
            FilteredAreaSeries.Fill.GradientFill = GradientFill.Solid;
            FilteredAreaSeries.Title.Text = "P(f)";
            FilteredAreaSeries.LimitYToStackSegment = true;
            _ChartArea.ViewXY.AreaSeries.Add(FilteredAreaSeries);
            
            _SignalGenerator.SamplingFrequency = 40000;
            //Add random noises to singal generator
            _SignalGenerator.WaveformRandomNoises.Add(new RandomNoiseComponent()
            {
                Amplitude = 30,
            });

            //Update select signals list
            SelectSignals.ItemsSource = new List<string>()
            {
                "Chirp (freq. sweep)",
                "1500Hz sine and 3500Hz sine",
                "Multiple square signals",
                "Multiple triangle signals",
                "Mixed sine, triangle, square"
            };
            //Select first item
            SelectSignals.SelectedIndex = 0;
            _Chart.EndUpdate();
            _ChartArea.EndUpdate();
        }

        private void ViewXY_Zoomed(object sender, ZoomedXYEventArgs e)
        {
            e.CancelRendering = true; 
            
            _Chart.BeginUpdate();

            //Set range from -range length to 0, static, X axis. 
            double rangeS = e.XRanges[0].CurrentMax - e.XRanges[0].CurrentMin; 
            _Chart.ViewXY.XAxes[1].SetRange(-rangeS, 0);

            _Chart.EndUpdate(); 
        }

        /// <summary>
        /// Range Y changed
        /// </summary>
        /// <param name="sender">y axis</param>
        /// <param name="e">RangeChangedEventArgs</param>
        private void SpectrumYAxisRangeChange(object sender, RangeChangedEventArgs e)
        {
            _ChartArea.BeginUpdate();
            if(e.NewMin < 0)
                e.NewMin = 0;

            if (e.NewMax != _ChartArea.ViewXY.YAxes[1].Maximum || e.NewMin != _ChartArea.ViewXY.YAxes[1].Minimum)
            {
                if (_ChartArea.ViewXY.YAxes[1].Minimum != e.NewMin && e.NewMin >= 0)
                    _ChartArea.ViewXY.YAxes[1].Minimum = e.NewMin;
                if (_ChartArea.ViewXY.YAxes[1].Maximum != e.NewMax)
                    _ChartArea.ViewXY.YAxes[1].Maximum = e.NewMax;
            }
            if (e.NewMax != _ChartArea.ViewXY.YAxes[0].Maximum || e.NewMin != _ChartArea.ViewXY.YAxes[0].Minimum)
            {
                if (_ChartArea.ViewXY.YAxes[0].Minimum != e.NewMin && e.NewMin >= 0)
                    _ChartArea.ViewXY.YAxes[0].Minimum = e.NewMin;
                if (_ChartArea.ViewXY.YAxes[0].Maximum != e.NewMax)
                    _ChartArea.ViewXY.YAxes[0].Maximum = e.NewMax;
            }
            _ChartArea.EndUpdate();
        }

        /// <summary>
        /// Dispose
        /// </summary>
        public void Dispose()
        {
            if (_SignalGenerator.IsEnabled)
            {
                _SignalGenerator.StopRequest();
                _SignalGenerator.DataGenerated -= _SignalGenerator_DataGenerated;
            }
            _chartUpdater = null;
            if (_Chart != null)
            {
                _Chart.Dispose();
                _Chart = null;
            }
            if (_spectrumCalculator != null)
            {
                _spectrumCalculator = null;
            }
        }

        /// <summary>
        /// Slider values change
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SliderValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if(e.OldValue == e.NewValue || _SignalGenerator is null)
                return;

            Slider sli = sender as Slider;

            switch (sli.Name)
            {
                case "Amplitude":
                    _SignalGenerator.WaveformFrequencySweepSines[0].Amplitude = e.NewValue;
                    break;

                case "FrequencyFrom":
                    _SignalGenerator.WaveformFrequencySweepSines[0].FrequencyFrom = e.NewValue;
                    break;

                case "FrequencyTo":
                    _SignalGenerator.WaveformFrequencySweepSines[0].FrequencyTo = e.NewValue;
                    break;

                case "DurationMs":
                    _SignalGenerator.WaveformFrequencySweepSines[0].DurationMs = e.NewValue;
                    break;
                case "SelectSmooth":
                    _spectrumSmoothLength = (int)e.NewValue;
                    break;
            }


        }

        /// <summary>
        /// Enable generator controls
        /// </summary>
        /// <param name="enabled"></param>
        private void Controls(bool enabled)
        {
            GenVisibility =  enabled ? Visibility.Visible: Visibility.Collapsed;
        }

        /// <summary>
        /// Signal selection changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SelectSignals_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBox cd = sender as ComboBox;
            _SignalGenerator.WaveformFrequencySweepSines.Clear();
            _SignalGenerator.WaveformAmplitudeSweepSines.Clear();
            _SignalGenerator.WaveformSines.Clear();
            _SignalGenerator.WaveformSquares.Clear();
            _SignalGenerator.WaveformTriangles.Clear();
            switch (cd.SelectedIndex)
            {
                //Frequency sweep sines
                case 0:
                    _SignalGenerator.WaveformFrequencySweepSines.Add(new FrequencySineSweepComponent()
                    {
                        Amplitude = 1000,
                        DurationMs = 1000,
                        FrequencyFrom = 0,
                        FrequencyTo = 5000
                    });
                    Amplitude.Value = _SignalGenerator.WaveformFrequencySweepSines[0].Amplitude;
                    FrequencyFrom.Value = _SignalGenerator.WaveformFrequencySweepSines[0].FrequencyFrom;
                    FrequencyTo.Value = _SignalGenerator.WaveformFrequencySweepSines[0].FrequencyTo;
                    DurationMs.Value = _SignalGenerator.WaveformFrequencySweepSines[0].DurationMs;
                    
                    Controls(true);
                    break;
                //Sines 2x
                case 1:
                    _SignalGenerator.WaveformSines.Add(new SineComponent()
                    {
                        Amplitude = 1000,
                        Frequency = 1500

                    });
                    _SignalGenerator.WaveformSines.Add(new SineComponent()
                    {
                        Amplitude = 1000,
                        Frequency = 3500
                    });
                    
                    Controls(false);
                    break;
                //Squares 3x
                case 2:
                    _SignalGenerator.WaveformSquares.Add(new SquareComponent()
                    {
                        Amplitude = 1000,
                        Frequency = 1800,
                        Symmetry = 0.7
                    });
                    _SignalGenerator.WaveformSquares.Add(new SquareComponent()
                    {
                        Amplitude = 1000,
                        Frequency = 2500,
                        Symmetry = 0.4
                    });
                    _SignalGenerator.WaveformSquares.Add(new SquareComponent()
                    {
                        Amplitude = 1000,
                        Frequency = 4200,
                        Symmetry = 0.5
                    });
                    
                    Controls(false);
                    break;
                //Triangles 3x
                case 3:
                    _SignalGenerator.WaveformTriangles.Add(new TriangleComponent()
                    {
                        Amplitude = 1000,
                        Symmetry = 0.1,
                        Frequency = 1000
                    });
                    _SignalGenerator.WaveformTriangles.Add(new TriangleComponent()
                    {
                        Amplitude = 1000,
                        Symmetry = 0.7,
                        Frequency = 2500
                    });
                    _SignalGenerator.WaveformTriangles.Add(new TriangleComponent()
                    {
                        Amplitude = 1000,
                        Symmetry = 0.5,
                        Frequency = 4200
                    });
                    
                    Controls(false);
                    break;
                //Triangles Squares Sines FrequencySweepSine
                case 4:
                    _SignalGenerator.WaveformTriangles.Add(new TriangleComponent()
                    {
                        Amplitude = 1000,
                        Symmetry = 0.5,
                        Frequency = 4200
                    });
                    _SignalGenerator.WaveformSquares.Add(new SquareComponent()
                    {
                        Amplitude = 1000,
                        Frequency = 2500,
                        Symmetry = 0.4
                    });
                    _SignalGenerator.WaveformSines.Add(new SineComponent()
                    {
                        Amplitude = 1000,
                        Frequency = 1500

                    });
                    _SignalGenerator.WaveformFrequencySweepSines.Add(new FrequencySineSweepComponent()
                    {
                        Amplitude = 1000,
                        DurationMs = 5000,
                        FrequencyFrom = 0,
                        FrequencyTo = 5000
                    });

                    Controls(false);
                    break;
            }
            FitSignalsY();
            FitSpectrumY();
        }
        
        /// <summary>
        /// Fir filter lowPass
        /// </summary>
        double[] _FirLowPass = new double[]
{
0.004226050850621026   ,
-0.0011206662962690004 ,
-0.001113450186640089  ,
-0.0011559854489163691 ,
-0.001202715200594507  ,
-0.0012101103348380274 ,
-0.0011425531870766375 ,
-0.0009767749862591418 ,
-0.0007064695323197611 ,
-0.0003438034560027987 ,
0.00008128424275598237 ,
0.000523476260739363   ,
0.0009287206974764959  ,
0.0012405022110333487  ,
0.001408766352516787   ,
0.0013976694319938025  ,
0.0011915188755995787  ,
0.0008005684302539314  ,
0.00025974198861247787 ,
-0.00037050465629736445,
-0.0010126697841307446 ,
-0.0015806333386465872 ,
-0.001990707975256521  ,
-0.0021738248754936363 ,
-0.002084222449793219  ,
-0.001709395813362088  ,
-0.0010766932930313892 ,
-0.0002461033090471316 ,
0.0006859128196021152  ,
0.0016044595762775419  ,
0.002385419976189769   ,
0.002913230408900621   ,
0.0030967783375129934  ,
0.0028831791897421717  ,
0.0022687276962622452  ,
0.001302658243028626   ,
0.00008507914648069891 ,
-0.0012422901737049671 ,
-0.002512269831862788  ,
-0.003551741768118783  ,
-0.00420457485556677   ,
-0.004354118027994228  ,
-0.003938321861739749  ,
-0.0029716820111236287 ,
-0.001531907006483214  ,
0.00022407057646220353 ,
0.0020905002155328263  ,
0.0038320493974374247  ,
0.0052089056628852924  ,
0.006009771029304396   ,
0.0060820818640794395  ,
0.005357008824590339   ,
0.0038651336219344048  ,
0.0017384194952183366  ,
-0.0007966360423090588 ,
-0.0034423740069433927 ,
-0.005861259875423409  ,
-0.007716540042762884  ,
-0.008715886043257771  ,
-0.008651832356698091  ,
-0.007437001242608155  ,
-0.005124229456264983  ,
-0.0019133285176271718 ,
0.00186187160454602    ,
0.0057645665573405665  ,
0.009300279510596171   ,
0.011972898527576813   ,
0.01333974802412681    ,
0.013065710013436668   ,
0.011027568071266768   ,
0.007259556324177315   ,
0.0020462433002381916  ,
-0.004126947197171483  ,
-0.010594299965463962  ,
-0.01656737135963027   ,
-0.02120874159608462   ,
-0.02371543439799458   ,
-0.02340569785935109   ,
-0.019798502827123057  ,
-0.012676887806182386  ,
-0.0021295291929309236 ,
0.011437657565632186   ,
0.02731931735269941    ,
0.04455916100291758    ,
0.0620258328439608     ,
0.07850512518968958    ,
0.09280922068685027    ,
0.10387740222647217    ,
0.11087521497264087    ,
0.11326914418191589    ,
0.11087521497264087    ,
0.10387740222647217    ,
0.09280922068685027    ,
0.07850512518968958    ,
0.0620258328439608     ,
0.04455916100291758    ,
0.02731931735269941    ,
0.011437657565632186   ,
-0.0021295291929309236 ,
-0.012676887806182386  ,
-0.019798502827123057  ,
-0.02340569785935109   ,
-0.02371543439799458   ,
-0.02120874159608462   ,
-0.01656737135963027   ,
-0.010594299965463962  ,
-0.004126947197171483  ,
0.0020462433002381916  ,
0.007259556324177315   ,
0.011027568071266768   ,
0.013065710013436668   ,
0.01333974802412681    ,
0.011972898527576813   ,
0.009300279510596171   ,
0.0057645665573405665  ,
0.00186187160454602    ,
-0.0019133285176271718 ,
-0.005124229456264983  ,
-0.007437001242608155  ,
-0.008651832356698091  ,
-0.008715886043257771  ,
-0.007716540042762884  ,
-0.005861259875423409  ,
-0.0034423740069433927 ,
-0.0007966360423090588 ,
0.0017384194952183366  ,
0.0038651336219344048  ,
0.005357008824590339   ,
0.0060820818640794395  ,
0.006009771029304396   ,
0.0052089056628852924  ,
0.0038320493974374247  ,
0.0020905002155328263  ,
0.00022407057646220353 ,
-0.001531907006483214  ,
-0.0029716820111236287 ,
-0.003938321861739749  ,
-0.004354118027994228  ,
-0.00420457485556677   ,
-0.003551741768118783  ,
-0.002512269831862788  ,
-0.0012422901737049671 ,
0.00008507914648069891 ,
0.001302658243028626   ,
0.0022687276962622452  ,
0.0028831791897421717  ,
0.0030967783375129934  ,
0.002913230408900621   ,
0.002385419976189769   ,
0.0016044595762775419  ,
0.0006859128196021152  ,
-0.0002461033090471316 ,
-0.0010766932930313892 ,
-0.001709395813362088  ,
-0.002084222449793219  ,
-0.0021738248754936363 ,
-0.001990707975256521  ,
-0.0015806333386465872 ,
-0.0010126697841307446 ,
-0.00037050465629736445,
0.00025974198861247787 ,
0.0008005684302539314  ,
0.0011915188755995787  ,
0.0013976694319938025  ,
0.001408766352516787   ,
0.0012405022110333487  ,
0.0009287206974764959  ,
0.000523476260739363   ,
0.00008128424275598237 ,
-0.0003438034560027987 ,
-0.0007064695323197611 ,
-0.0009767749862591418 ,
-0.0011425531870766375 ,
-0.0012101103348380274 ,
-0.001202715200594507  ,
-0.0011559854489163691 ,
-0.001113450186640089  ,
-0.0011206662962690004 ,
0.004226050850621026


};
        /// <summary>
        /// Fir filter HighPass
        /// </summary>
        double[] _FirHighPass = new double[]
{
0.0007718706214085175  ,
0.0026353575784694057  ,
0.0005868734590722826  ,
0.0012765216890619061  ,
0.0010042124334418     ,
0.0008776595428446031  ,
0.0005689798932533497  ,
0.00020216390182821884 ,
-0.0002167513704788519 ,
-0.0006335712049681822 ,
-0.0009987400257732717 ,
-0.0012638119040960588 ,
-0.0013840811504051277 ,
-0.0013351676861438175 ,
-0.001104929464135559  ,
-0.0007114262067432568 ,
-0.00018642008740215495,
0.0004093713379809899  ,
0.0010083336007632655  ,
0.0015310521726760087  ,
0.0019027232807317703  ,
0.0020639868927346426  ,
0.0019715892289224736  ,
0.0016193301730507168  ,
0.0010262826552115236  ,
0.00025064373781328366 ,
-0.0006233066385588629 ,
-0.001493352611195355  ,
-0.0022441262011007245 ,
-0.002770745596894636  ,
-0.0029870340837763803 ,
-0.002836980652582105  ,
-0.0023124920424897758 ,
-0.0014489566820735135 ,
-0.0003273672890142949 ,
0.0009253592545053427  ,
0.0021621453551439198  ,
0.0032231581481478814  ,
0.003956055937758613   ,
0.004242260796501958   ,
0.004009824606186083   ,
0.0032449266533948642  ,
0.002003072061163469   ,
0.000406994834800761   ,
-0.001370420057191982  ,
-0.003116899561025202  ,
-0.0046044559342214705 ,
-0.00562303720888033   ,
-0.00600619670004049   ,
-0.005652286713904704  ,
-0.004544981903171214  ,
-0.0027633378124282846 ,
-0.00047972613863189425,
0.002057481388203122   ,
0.00454824932146451    ,
0.00667082756191034    ,
0.008122821027917945   ,
0.00865967844735318    ,
0.00813250741630566    ,
0.00651268154394475    ,
0.0039038332073708997  ,
0.000543854298147739   ,
-0.003213120820591729  ,
-0.006929638157873348  ,
-0.010129731740685996  ,
-0.012354776313218229  ,
-0.01322013356823554   ,
-0.012466359934370832  ,
-0.010004097217075488  ,
-0.005941860432299477  ,
-0.0005933660784473592 ,
0.005535893502631852   ,
0.011783460785723014   ,
0.0173870710764547     ,
0.02155393085722103    ,
0.02353886783918028    ,
0.022726079406010718   ,
0.0186973395045955     ,
0.01128817179905101    ,
0.0006240582460040155  ,
-0.012873370953255974  ,
-0.02850696095533895   ,
-0.04535069811082266   ,
-0.06232148103524391   ,
-0.07826750505753437   ,
-0.09206579311761466   ,
-0.10271952019272598   ,
-0.10944596232255124   ,
0.8882545040240198     ,
-0.10944596232255124   ,
-0.10271952019272598   ,
-0.09206579311761466   ,
-0.07826750505753437   ,
-0.06232148103524391   ,
-0.04535069811082266   ,
-0.02850696095533895   ,
-0.012873370953255974  ,
0.0006240582460040155  ,
0.01128817179905101    ,
0.0186973395045955     ,
0.022726079406010718   ,
0.02353886783918028    ,
0.02155393085722103    ,
0.0173870710764547     ,
0.011783460785723014   ,
0.005535893502631852   ,
-0.0005933660784473592 ,
-0.005941860432299477  ,
-0.010004097217075488  ,
-0.012466359934370832  ,
-0.01322013356823554   ,
-0.012354776313218229  ,
-0.010129731740685996  ,
-0.006929638157873348  ,
-0.003213120820591729  ,
0.000543854298147739   ,
0.0039038332073708997  ,
0.00651268154394475    ,
0.00813250741630566    ,
0.00865967844735318    ,
0.008122821027917945   ,
0.00667082756191034    ,
0.00454824932146451    ,
0.002057481388203122   ,
-0.00047972613863189425,
-0.0027633378124282846 ,
-0.004544981903171214  ,
-0.005652286713904704  ,
-0.00600619670004049   ,
-0.00562303720888033   ,
-0.0046044559342214705 ,
-0.003116899561025202  ,
-0.001370420057191982  ,
0.000406994834800761   ,
0.002003072061163469   ,
0.0032449266533948642  ,
0.004009824606186083   ,
0.004242260796501958   ,
0.003956055937758613   ,
0.0032231581481478814  ,
0.0021621453551439198  ,
0.0009253592545053427  ,
-0.0003273672890142949 ,
-0.0014489566820735135 ,
-0.0023124920424897758 ,
-0.002836980652582105  ,
-0.0029870340837763803 ,
-0.002770745596894636  ,
-0.0022441262011007245 ,
-0.001493352611195355  ,
-0.0006233066385588629 ,
0.00025064373781328366 ,
0.0010262826552115236  ,
0.0016193301730507168  ,
0.0019715892289224736  ,
0.0020639868927346426  ,
0.0019027232807317703  ,
0.0015310521726760087  ,
0.0010083336007632655  ,
0.0004093713379809899  ,
-0.00018642008740215495,
-0.0007114262067432568 ,
-0.001104929464135559  ,
-0.0013351676861438175 ,
-0.0013840811504051277 ,
-0.0012638119040960588 ,
-0.0009987400257732717 ,
-0.0006335712049681822 ,
-0.0002167513704788519 ,
0.00020216390182821884 ,
0.0005689798932533497  ,
0.0008776595428446031  ,
0.0010042124334418     ,
0.0012765216890619061  ,
0.0005868734590722826  ,
0.0026353575784694057  ,
0.0007718706214085175  



};
        /// <summary>
        /// Fir filter BandBass
        /// </summary>
        double[] _FirBandPass = new double[]
{
0.005385977857049973    ,
0.001190880001305358    ,
0.0006633734876837097   ,
-0.00019394259269893654 ,
-0.0011225063656940023  ,
-0.001812087387769884   ,
-0.0020142486954373012  ,
-0.001645840863176788   ,
-0.000851351081828534   ,
0.000009839239712166009 ,
0.00046365748643209867  ,
0.00008803753124807572  ,
-0.0013078835071557346  ,
-0.003574676200967945   ,
-0.006205231437382682   ,
-0.008390831074261343   ,
-0.009309005291391396   ,
-0.008332391660026436   ,
-0.005304863607720231   ,
-0.0006533426960113092  ,
0.004662544434494776    ,
0.00938232522673989     ,
0.012306987044100147    ,
0.012690619488021635    ,
0.010535475058893967    ,
0.0066762368853928175   ,
0.002590763752137642    ,
-0.000017120938960995262,
0.00022093000671056316  ,
0.003789344115737846    ,
0.00997554952568987     ,
0.01685997921310099     ,
0.021722490353651924    ,
0.02175686144991622     ,
0.01493553502340815     ,
0.0007839591970715715   ,
-0.019175381876427307   ,
-0.041480236805132034   ,
-0.06133427528258058    ,
-0.07365744840312739    ,
-0.07433043802861765    ,
-0.06134186366524776    ,
-0.035505225810424465   ,
-0.0005330710459627332  ,
0.0375771498930247      ,
0.07176371802792057     ,
0.09541997706138233     ,
0.10386801503017322     ,
0.09541997706138233     ,
0.07176371802792057     ,
0.0375771498930247      ,
-0.0005330710459627332  ,
-0.035505225810424465   ,
-0.06134186366524776    ,
-0.07433043802861765    ,
-0.07365744840312739    ,
-0.06133427528258058    ,
-0.041480236805132034   ,
-0.019175381876427307   ,
0.0007839591970715715   ,
0.01493553502340815     ,
0.02175686144991622     ,
0.021722490353651924    ,
0.01685997921310099     ,
0.00997554952568987     ,
0.003789344115737846    ,
0.00022093000671056316  ,
-0.000017120938960995262,
0.002590763752137642    ,
0.0066762368853928175   ,
0.010535475058893967    ,
0.012690619488021635    ,
0.012306987044100147    ,
0.00938232522673989     ,
0.004662544434494776    ,
-0.0006533426960113092  ,
-0.005304863607720231   ,
-0.008332391660026436   ,
-0.009309005291391396   ,
-0.008390831074261343   ,
-0.006205231437382682   ,
-0.003574676200967945   ,
-0.0013078835071557346  ,
0.00008803753124807572  ,
0.00046365748643209867  ,
0.000009839239712166009 ,
-0.000851351081828534   ,
-0.001645840863176788   ,
-0.0020142486954373012  ,
-0.001812087387769884   ,
-0.0011225063656940023  ,
-0.00019394259269893654 ,
0.0006633734876837097   ,
0.001190880001305358    ,
0.005385977857049973    
                        

};
        /// <summary>
        /// Fir filter Band stop
        /// </summary>
        double[] _FirBandStop = new double[]
{
-0.002524173934913049     ,
-0.0013907888073557521    ,
-0.0013559685530776366    ,
-0.0010318197293189202    ,
-0.000504105313998265     ,
0.0000498007482856444     ,
0.0004174516491426809     ,
0.0004297493311464823     ,
0.00003227599664825799    ,
-0.0006616342418485425    ,
-0.0013629204430361424    ,
-0.0016953748715860938    ,
-0.0012871752653008078    ,
0.00006973384105288334    ,
0.0023155794442226638     ,
0.005073475415634717      ,
0.007702731230088818      ,
0.009445954223958495      ,
0.009645262409868119      ,
0.007962095855093594      ,
0.004527444232315727      ,
-0.00002344138836581316   ,
-0.004683787670500806     ,
-0.008333093118380232     ,
-0.010073669783826202     ,
-0.009537236390855453     ,
-0.00706476755416158      ,
-0.003678923165367448     ,
-0.0008323191101798652    ,
0.000022686441672209063   ,
-0.002064789912794912     ,
-0.007127234731724314     ,
-0.014078119781738915     ,
-0.02083351234260769      ,
-0.024759882811491184     ,
-0.02336866821966677      ,
-0.015086023433871021     ,
0.00011059436656393554    ,
0.020389214602628086      ,
0.04222081264639156       ,
0.06100773663494012       ,
0.07207436951337366       ,
0.0717914303463952        ,
0.05856335606221795       ,
0.03341137906289748       ,
-0.000020566662958225554  ,
-0.036079161915959175     ,
-0.06821808446086863      ,
-0.09037168143653038      ,
0.9017311814044645        ,
-0.09037168143653038      ,
-0.06821808446086863      ,
-0.036079161915959175     ,
-0.000020566662958225554  ,
0.03341137906289748       ,
0.05856335606221795       ,
0.0717914303463952        ,
0.07207436951337366       ,
0.06100773663494012       ,
0.04222081264639156       ,
0.020389214602628086      ,
0.00011059436656393554    ,
-0.015086023433871021     ,
-0.02336866821966677      ,
-0.024759882811491184     ,
-0.02083351234260769      ,
-0.014078119781738915     ,
-0.007127234731724314     ,
-0.002064789912794912     ,
0.000022686441672209063   ,
-0.0008323191101798652    ,
-0.003678923165367448     ,
-0.00706476755416158      ,
-0.009537236390855453     ,
-0.010073669783826202     ,
-0.008333093118380232     ,
-0.004683787670500806     ,
-0.00002344138836581316   ,
0.004527444232315727      ,
0.007962095855093594      ,
0.009645262409868119      ,
0.009445954223958495      ,
0.007702731230088818      ,
0.005073475415634717      ,
0.0023155794442226638     ,
0.00006973384105288334    ,
-0.0012871752653008078    ,
-0.0016953748715860938    ,
-0.0013629204430361424    ,
-0.0006616342418485425    ,
0.00003227599664825799    ,
0.0004297493311464823     ,
0.0004174516491426809     ,
0.0000498007482856444     ,
-0.000504105313998265     ,
-0.0010318197293189202    ,
-0.0013559685530776366    ,
-0.0013907888073557521    ,
-0.002524173934913049

};
        /// <summary>
        /// Checkbox checked
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            if((sender as CheckBox).IsChecked == true)
            {
                _SignalGenerator.WaveformRandomNoises[0].Enabled = false;
            }
            else
            {
                _SignalGenerator.WaveformRandomNoises[0].Enabled = true;
            }
        }

        /// <summary>
        /// Button click (filter settings)
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Button_Click(object sender, RoutedEventArgs e)
        {
            Button b = (sender as Button);
            switch (b.Name)
            {
                case "LowPassFIR":
                    _FIRFilter.SetFactors(_FirLowPass);
                    FirIsInUse = true;
                    break;

                case "HighPassFIR":
                    _FIRFilter.SetFactors(_FirHighPass);
                    FirIsInUse = true;
                    break;

                case "BandPassFIR":
                    _FIRFilter.SetFactors(_FirBandPass);
                    FirIsInUse = true;
                    break;

                case "BandStopFIR":
                    _FIRFilter.SetFactors(_FirBandStop);
                    FirIsInUse = true;
                    break;

                case "LowPassIIR":
                    _IIRFilter.SetABFactors(new double[] {
                        0.4040290988223374,-1.979297129242377,3.6857057965464457,-3.1010890119996937,1},
                        new double[] {1,4,6,4,1},
                        -10000000000, 10000000000, 1711.4579957006445);
                    FirIsInUse = false;
                    break;

                case "HighPassIIR":
                    _IIRFilter.SetABFactors(new double[] {
                        0.4040290988223374, -1.979297129242377, 3.6857057965464457, -3.1010890119996937, 1 },
                        new double[] { 1, -4, 6, -4, 1 },
                        -1000000, 1000000, 1.5732359469865198);
                    FirIsInUse = false;
                    break;

                case "BandPassIIR":
                    _IIRFilter.SetABFactors(new double[] {
                        0.3526117584690343, -3.0421924676490857, 11.669919973082614, -25.98705536028583, 36.735346210579735, -33.75156146228977, 19.680247615846543, -6.657278361093047, 1 },
                        new double[] { 1, 0, -4, 0, 6, 0, -4, 0, 1 },
                        -100000, 100000, 1041.215844239782);
                    FirIsInUse = false;
                    break;

                case "BandStopIIR":
                    _IIRFilter.SetABFactors(new double[] {
                        0.7089242326314994, -5.86388306309268, 21.2791333528488, -44.24676850474157, 57.66076881828131, -48.22216315053005, 25.274254581644097, -7.590266192033257, 1 },
                        new double[] { 1, -7.930894575990032, 27.587158290800538, -54.97056135322481, 68.62859536591444, -54.97056135322481, 27.58715829080054, -7.930894575990032, 1 },
                        -100000, 100000, 1.1876817665910695);
                    FirIsInUse = false;
                    break;
            }
        }

        /// <summary>
        /// Zoom buttons functions
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SignalZoom(object sender, RoutedEventArgs e)
        {
            switch((sender as Button).Tag)
            {
                case "In":
                if(_Chart.ViewXY.XAxes[0].Maximum > (_Chart.ViewXY.XAxes[0].Maximum - _Chart.ViewXY.XAxes[0].Minimum) / 2) 
                    { 
                        _Chart.ViewXY.XAxes[0].Minimum += (_Chart.ViewXY.XAxes[0].Maximum - _Chart.ViewXY.XAxes[0].Minimum)/2;
                        _Chart.ViewXY.XAxes[1].Minimum += (_Chart.ViewXY.XAxes[1].Maximum - _Chart.ViewXY.XAxes[1].Minimum)/2;
                    }
                    break;
                case "Out":
                    _Chart.ViewXY.XAxes[0].Minimum -= (_Chart.ViewXY.XAxes[0].Maximum - _Chart.ViewXY.XAxes[0].Minimum)/2;
                    _Chart.ViewXY.XAxes[1].Minimum -= (_Chart.ViewXY.XAxes[1].Maximum - _Chart.ViewXY.XAxes[1].Minimum)/2;
                    break;
                case "SignalsFit":
                    FitSignalsY();
                    break;
                case "SpectrumFit":
                    FitSpectrumY();
                    break;
            }
        }

        /// <summary>
        /// FitSpectrum Y
        /// </summary>
        private void FitSpectrumY()
        {
            _ZoomSpectrumthread = new Thread(ThreadedZoom);
            _ZoomSpectrumthread.IsBackground = true;
            _ZoomSpectrumthread.Start();
        }

        /// <summary>
        /// Thread to do zoom operation 1s cap
        /// </summary>
        private void ThreadedZoom()
        {
            _highestSpectrumSpike = 0;
            Thread.Sleep(1000);
            Dispatcher.Invoke(() => 
            { 
                _ChartArea.BeginUpdate();
                _ChartArea.ViewXY.YAxes[0].SetRange(0, _highestSpectrumSpike);
                _ChartArea.ViewXY.XAxes[0].SetRange(0,5000);
                _ChartArea.EndUpdate();
            });
        }


        /// <summary>
        /// Fit signals Y
        /// </summary>
        private void FitSignalsY()
        {
            _Chart.BeginUpdate();
            double amplitudes = getAmplidutes();
            _Chart.ViewXY.YAxes[0].SetRange(-amplitudes, amplitudes);
            _Chart.ViewXY.YAxes[1].SetRange(-amplitudes, amplitudes);
            _Chart.EndUpdate();
        }

        /// <summary>
        /// Get amplitudes of all signals
        /// </summary>
        /// <returns>sum of amplitudes</returns>
        private double getAmplidutes()
        {
            double overal = 0;
            foreach (AmplitudeSineSweepComponent aswc in _SignalGenerator.WaveformAmplitudeSweepSines)
            {
                overal += aswc.AmplitudeTo - aswc.AmplitudeFrom;
            }
            foreach (FrequencySineSweepComponent fssc in _SignalGenerator.WaveformFrequencySweepSines)
            {
                overal += fssc.Amplitude;
            }
            foreach (SineComponent sc in _SignalGenerator.WaveformSines)
            {
                overal += sc.Amplitude;
            }
            foreach (SquareComponent sc in _SignalGenerator.WaveformSquares)
            {
                overal += sc.Amplitude;
            }
            foreach (TriangleComponent tc in _SignalGenerator.WaveformTriangles)
            {
                overal += tc.Amplitude;
            }
            foreach (RandomNoiseComponent rnc in _SignalGenerator.WaveformRandomNoises)
            {
                overal += rnc.Amplitude;
            }
            return overal;
        }
    }

    /// <summary>
    /// 1 decimal rounding
    /// </summary>
    public class RoundMyNumberConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object paramater, CultureInfo culture)
        {
            double result = 0;
            if (value != null && (double.TryParse(value.ToString(), out result)))
                return Math.Round(result, 1);
            else
                return value;
        }

        public object ConvertBack(object value, Type targetType, object paramater, CultureInfo culture)
        {
            return value;
        }
    }
    /// <summary>
    /// 4 decimals rounding
    /// </summary>
    public class RoundMyNumberConverter4Decimals : IValueConverter
    {
        public object Convert(object value, Type targetType, object paramater, CultureInfo culture)
        {
            double result = 0;
            if (value != null && (double.TryParse(value.ToString(), out result)))
                return Math.Round(result, 4);
            else
                return value;
        }

        public object ConvertBack(object value, Type targetType, object paramater, CultureInfo culture)
        {
            return value;
        }
    }
}
